/*==============================================================================
 LQTMENU class
==============================================================================*/
#include "hbqtgui.ch"
#include "hbclass.ch"
#include "common.ch"

#define MNU_QBAR        0
#define MNU_QACT        1
#define MNU_QMNU        2
#define MNU_QSEP        3

CLASS LQTMENU FROM CONTROL

// shared data
   CLASS VAR oCurrMnuParent                        INIT NIL      SHARED      // current popup menu parent for item or new popup
   CLASS VAR oSuMnuParent                          INIT NIL      SHARED      // super parent in other words mainmenu; must be the same

   DATA cClassName                                 INIT "LQTMENU"

// data: please preserve alfabetic order.
   DATA bAction                                    INIT NIL      PROTECTED
   DATA cCaption                                   INIT NIL      PROTECTED
   DATA lChecked                                   INIT NIL      PROTECTED
   DATA lEnabled                                   INIT .T.      PROTECTED
   DATA cImage                                     INIT NIL      PROTECTED
   DATA nMnuType                                   INIT NIL      PROTECTED
   DATA cName                                      INIT NIL      PROTECTED
   DATA cTitle                                     INIT NIL      PROTECTED
   DATA lVisible                                   INIT .T.      PROTECTED

// method: please preserve alfabetic order. New and Create must be the first
   METHOD New
   METHOD Create
   METHOD Action                                   SETGET
   METHOD Caption                                  SETGET
   METHOD Checked                                  SETGET
   METHOD ChildList                                INLINE ::aControls
   METHOD Enabled                                  SETGET
   METHOD FontSize                                 SETGET // overwrite basic method
   METHOD GetName
   METHOD Image                                    SETGET
   METHOD MnuType                                  SETGET
   METHOD Title                                    SETGET
   METHOD Visible                                  SETGET

ENDCLASS

/*..............................................................................
 New
..............................................................................*/
METHOD New( cName ) CLASS LQTMENU

// set-up a default name
   ::cName        := IIF( VALTYPE( cName) == "C", cName, ::GetName() )

RETURN Self

/*..............................................................................
 Create
..............................................................................*/
METHOD Create() CLASS LQTMENU

   LOCAL nX

   ::lCreated  := .T.

   IF ::nMnuType == MNU_QBAR
//      ::oParent:oMainMenu := ::oSuMnuParent
      ::oParent:SetUpMenu( ::oSuMnuParent )
      ::oQtObject := QMenuBar( ::oParent:oQtObject )
      ::oFont := ::oParent:oQtObject:Font()
      IF VALTYPE( ::nFontSize )        <> 'U'
         IF ::oFont:PointSize == -1; ::oFont:setPixelSize( ::nFontSize ); ELSE; ::oFont:setPointSize( ::nFontSize ); ENDIF
         ::oQTObject:SetFont( ::oFont )
      ENDIF
   ENDIF

   IF ::nMnuType == MNU_QMNU
      ::oQtObject := QMenu( ::oParent:oQtObject )
      ::oFont := ::oParent:oQtObject:Font()
      ::oQtObject:setTitle( ::cTitle )
      IF hb_IsChar( ::cImage ) == .T.
         ::oQtObject:setIcon( ::cImage )
      ENDIF
   ENDIF

   IF ::nMnuType == MNU_QACT
      ::oQtObject := QAction( ::oParent:oQtObject )
      ::oFont := ::oParent:oQtObject:Font()
      ::oQtObject:setText( ::cCaption )
      IF hb_IsChar( ::cImage ) == .T.
         ::oQtObject:setIcon( ::cImage )
      ENDIF
      IF hb_IsLogical( ::lChecked ) == .T.
         ::oQtObject:setCheckable( .T. )
         ::oQtObject:setChecked( ::lChecked )
      ENDIF
      IF hb_IsLogical( ::lEnabled ) == .T.
         ::oQtObject:setEnabled( ::lEnabled )
      ENDIF
      IF hb_IsLogical( ::lVisible ) == .T.
         ::oQtObject:setVisible( ::lVisible )
      ENDIF
      IF hb_IsBlock( ::bAction ) == .T.
         ::oQtObject:connect( "triggered(bool)", ::bAction )
      ENDIF
   ENDIF

   IF ::nMnuType == MNU_QSEP
      ::oQtObject := QAction( ::oParent:oQtObject )
      ::oFont := ::oParent:oQtObject:Font()
      ::oQtObject:setText( ::cCaption )
      ::oQtObject:setSeparator( .T. )
   ENDIF

   ::oQTObject:SetFont( ::oFont )

   FOR nX := 1 TO LEN( ::aControls )
      IF __objHasMethod( ::aControls[nX], "create" ) == .T.
         ::aControls[nX]:Create()
      ENDIF
   NEXT nX

   IF ::nMnuType == MNU_QBAR
      ::oParent:oQtObject:setMenuBar( ::oQtObject )
      // create Hmg3 compatibility referer
      FOR nX := 1 TO LEN( ::oSuMnuParent:aHmg3Comp )
         // add HmgObject to its parent
//         ::oSuMnuParent:oParent:AddToParent( ::oSuMnuParent:aHmg3Comp[nX] )
         AADD( ::oSuMnuParent:oParent:aControls,  ::oSuMnuParent:aHmg3Comp[nX]  )
         ::oSuMnuParent:oParent:AddData( ::oSuMnuParent:aHmg3Comp[nX]:cName, ::oSuMnuParent:aHmg3Comp[nX] )
      NEXT nX
   ENDIF
   IF ::nMnuType == MNU_QMNU
      ::oParent:oQtObject:addmenu( ::oQtObject )
   ENDIF
   IF ::nMnuType == MNU_QACT
      ::oParent:oQtObject:addaction( ::oQtObject )
   ENDIF
   IF ::nMnuType == MNU_QSEP
      ::oParent:oQtObject:addaction( ::oQtObject )
   ENDIF

RETURN Self

/*..............................................................................
 Action
..............................................................................*/
METHOD Action( bAction ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::bAction
   ELSEIF hb_IsBlock( bAction ) == .T.
      ::bAction := bAction
   ENDIF

RETURN Self

/*..............................................................................
 Caption
..............................................................................*/
METHOD Caption( cCaption ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::cCaption
   ELSEIF hb_IsChar( cCaption ) == .T.
      ::cCaption := cCaption
   ENDIF

RETURN NIL

/*..............................................................................
 Checked
..............................................................................*/
METHOD Checked( lChecked ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::lChecked
   ELSEIF hb_IsLogical( lChecked ) == .T.
      ::lChecked := lChecked
   ENDIF

RETURN NIL

/*..............................................................................
 Enabled
..............................................................................*/
METHOD Enabled( lEnabled ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::lEnabled
   ELSEIF hb_IsLogical( lEnabled ) == .T.
      ::lEnabled := lEnabled
   ENDIF

RETURN NIL

/*..............................................................................
 FontSize
..............................................................................*/
METHOD FontSize( nFontSize ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::nFontSize
   ELSEIF VALTYPE( nFontSize ) == "N"
      ::nFontSize      := nFontSize
   ENDIF

RETURN Self

/*..............................................................................
 GetName
..............................................................................*/
METHOD GetName() CLASS LQTMENU

   LOCAL cNewName
   
   ::oSuMnuParent:nMnuObjCounter += 1
   cNewName := "LQTMNU_" + hb_nTos( ::oSuMnuParent:nMnuObjCounter )
   
RETURN cNewName

/*..............................................................................
 Image
..............................................................................*/
METHOD Image( cImage ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::cImage
   ELSEIF hb_IsChar( cImage ) == .T.
      ::cImage := cImage
   ENDIF

RETURN NIL

/*..............................................................................
 MnuType
..............................................................................*/
METHOD MnuType( nMnuType ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::nMnuType
   ELSEIF hb_IsNumeric( nMnuType ) == .T.
      ::nMnuType := nMnuType
   ENDIF

RETURN NIL

/*..............................................................................
 Title
..............................................................................*/
METHOD Title( cTitle ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::cTitle
   ELSEIF hb_IsChar( cTitle ) == .T.
      ::cTitle := cTitle
   ENDIF

RETURN NIL

/*..............................................................................
 Visible
..............................................................................*/
METHOD Visible( lVisible ) CLASS LQTMENU

   IF PCOUNT() == 0
      RETURN ::lVisible
   ELSEIF hb_IsLogical( lVisible ) == .T.
      ::lVisible := lVisible
   ENDIF

RETURN NIL

/*==============================================================================
 LQTMENUBAR class
==============================================================================*/
CLASS LQTMENUBAR FROM LQTMENU

// method: please preserve alfabetic order.
   DATA aPopUpMenu               INIT {}      // to store popup level object, to retrieve previous or add a new
   DATA nMnuObjCounter           INIT 0       // to assign internal var name when cName is omitted: mainmenu is always 0, then add 1
   DATA aHmg3Comp                INIT {}      // to hanlde HMG3 compatibility for Item; syntax like this FormName.ItemMenu....

// method: please preserve alfabetic order. New and Create must be the first
   METHOD New
   METHOD AddMnuItem
   METHOD AddMnuPopUp
   METHOD AddMnuSeparator
   METHOD EndMnuMain
   METHOD EndMnuPopUp

ENDCLASS

/*..............................................................................
 New
..............................................................................*/
METHOD New( cName, oParent ) CLASS LQTMENUBAR

   ::aPopUpMenu      := {}
   ::nMnuObjCounter  := 0      // Menu object counter: 0 is always main menu
   ::aHmg3Comp       := {}
   ::cName           := IIF( VALTYPE( cName) == "C", cName, "LQTMNU_" + hb_ntos( ::nMnuObjCounter ) )
   ::oSuMnuParent    := Self
   ::oCurrMnuParent  := Self
   ::nMnuType        := MNU_QBAR
   
   IF hb_IsNumeric( oParent ) .AND. oParent = 0    // If numeric 0 ( zero ) force no parent.
      ::oParent    := NIL
   ELSEIF hb_IsObject( oParent )                   // If parent object is passed explicitly, use it.
      ::oParent    := oParent
   ELSEIF hb_IsObject( oParent := hb_qwith() )
      ::oParent    := oParent
   ELSE
      ::oParent := ::s_oDefaultParent
   ENDIF

// add HmgObject to its parent
//   ::oParent:AddToParent( Self )
   AADD( ::oParent:aControls , Self )
   ::oParent:AddData( ::cName , Self )

RETURN Self

/*..............................................................................
 AddMnuItem
..............................................................................*/
METHOD AddMnuItem( cName, cCaption, bAction, cImage, lChecked, lEnabled, lVisible ) CLASS LQTMENUBAR

   LOCAL oNewObject, cNewObjectName

// create an internal varname: don't move ::nMnuObjCounter++ inside hb_ntos. It's fails and I want count every element
   ::nMnuObjCounter++
   cNewObjectName := IIF( VALTYPE( cName) == "C", cName, "LQTMNU_" + hb_ntos( ::nMnuObjCounter ) )

   cCaption := IF( hb_IsChar( cCaption ) == .T., cCaption, cNewObjectName )

// create a new object with internal name
   oNewObject := LQTMENU():New( cNewObjectName )

// add created object to its parent
   __objAddData( ::oCurrMnuParent, cNewObjectName )
   ::oCurrMnuParent:&cNewObjectName := oNewObject
   AADD( ::oCurrMnuParent:aControls, oNewObject )

// setup HMG object
   ::oCurrMnuParent:&cNewObjectName:MnuType := MNU_QACT
   ::oCurrMnuParent:&cNewObjectName:oParent  := ::oCurrMnuParent
   ::oCurrMnuParent:&cNewObjectName:Caption := cCaption
   ::oCurrMnuParent:&cNewObjectName:Action  := bAction
   ::oCurrMnuParent:&cNewObjectName:Image   := cImage
   ::oCurrMnuParent:&cNewObjectName:Checked := lChecked
   ::oCurrMnuParent:&cNewObjectName:Enabled := lEnabled
   ::oCurrMnuParent:&cNewObjectName:Visible := lVisible

// create HMG3 data compatibility
   AADD( ::oSuMnuParent:aHmg3Comp, oNewObject )

RETURN NIL

/*..............................................................................
 AddMnuPopUp
..............................................................................*/
METHOD AddMnuPopUp( cName, cTitle, cImage ) CLASS LQTMENUBAR

   LOCAL oNewObject, cNewObjectName

// create an internal varname: don't move ::nMnuObjCounter++ inside hb_ntos. It's fails and I want count every element
   ::nMnuObjCounter++
   cNewObjectName := IIF( VALTYPE( cName) == "C", cName, "LQTMNU_" + hb_ntos( ::nMnuObjCounter ) )

   cTitle := IF( hb_IsChar( cTitle ) == .T., cTitle, cNewObjectName )

// create a new object with internal name
   oNewObject := LQTMENU():New( cNewObjectName )

// add created object to its parent
   __objAddData( ::oCurrMnuParent, cNewObjectName )
   ::oCurrMnuParent:&cNewObjectName := oNewObject
   AADD( ::oCurrMnuParent:aControls, oNewObject )

// setup HMG object
   ::oCurrMnuParent:&cNewObjectName:MnuType := MNU_QMNU
   ::oCurrMnuParent:&cNewObjectName:oParent  := ::oCurrMnuParent
   ::oCurrMnuParent:&cNewObjectName:Title   := cTitle
   ::oCurrMnuParent:&cNewObjectName:Image   := cImage

// now I am current parent
   ::oCurrMnuParent := ::oCurrMnuParent:&cNewObjectName
// create a new popup level
   AADD( ::oSuMnuParent:aPopUpMenu, ::oCurrMnuParent )

RETURN NIL

/*..............................................................................
 AddMnuSeparator
..............................................................................*/
METHOD AddMnuSeparator(cName, cCaption ) CLASS LQTMENUBAR

   LOCAL oNewObject, cNewObjectName

// create an internal varname: don't move ::nMnuObjCounter++ inside hb_ntos. It's fails and I want count every element
   ::nMnuObjCounter++
   cNewObjectName := IIF( VALTYPE( cName) == "C", cName, "LQTMNU_" + hb_ntos( ::nMnuObjCounter ) )

   cCaption := IF( hb_IsChar( cCaption ) == .T., cCaption, cNewObjectName )

// create a new object with internal name
   oNewObject := LQTMENU():New( cNewObjectName )

// add created object to its parent
   __objAddData( ::oCurrMnuParent, cNewObjectName )
   ::oCurrMnuParent:&cNewObjectName := oNewObject
   AADD( ::oCurrMnuParent:aControls, oNewObject )

// setup HMG object
   ::oCurrMnuParent:&cNewObjectName:MnuType := MNU_QSEP
   ::oCurrMnuParent:&cNewObjectName:oParent  := ::oCurrMnuParent
   ::oCurrMnuParent:&cNewObjectName:Caption := cCaption

RETURN NIL

/*..............................................................................
 EndMnuMain
..............................................................................*/
METHOD EndMnuMain() CLASS LQTMENUBAR

   ::oCurrMnuParent := ::oSuMnuParent

RETURN NIL

/*..............................................................................
 EndMnuPopUp
..............................................................................*/
METHOD EndMnuPopUp() CLASS LQTMENUBAR

   LOCAL nLast := LEN( ::oSuMnuParent:aPopUpMenu )

   IF nLast > 0
      ::oSuMnuParent:aPopUpMenu := hb_aDel( ::oSuMnuParent:aPopUpMenu, nLast, .T. ) // resize param
      nLast := LEN( ::oSuMnuParent:aPopUpMenu )
   ENDIF
   
   ::oCurrMnuParent := IIF( nLast > 0, ::oSuMnuParent:aPopUpMenu[nLast], ::oSuMnuParent )

RETURN NIL
