/*
 * $Id: window.prg 659 2011-07-06 17:37:29Z concentra $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"
#include "hbqtgui.ch"
#include "hmg.ch"

/*----------------------------------------------------------------------*/

CLASS WINDOW FROM VIRTUALWINDOW

//   CLASS VAR oDefaultMenuItem                     INIT   NIL           SHARED

   // Internal Data
   DATA oMainMenu                                 INIT   NIL      // need this to store menu: local action on Menu class, harbour trick
   DATA oNotifyMenu                               INIT   NIL
   DATA oSystemTrayIcon                           INIT   NIL
   DATA aNotifyItems                              INIT   {}
   DATA oStatusBar                                INIT   NIL
   DATA lExit                                     INIT   .F.
   DATA cClass                                    INIT   'WINDOW'
   DATA oEventLoop                                INIT   NIL
   DATA aPanels                                   INIT   {}
   DATA lActivated                                INIT   .F.
   DATA lMaxButton                                INIT   .T.
   DATA lMinButton                                INIT   .T.
   DATA lSysMenu                                  INIT   .T.
   DATA lTitleBar                                 INIT   .T.
   DATA bOnRelease                                INIT   NIL     // Im' sorry, code block must be NIL and not {|| .T. }
   DATA lShow                                     INIT   .T.
   DATA lTopmost                                  INIT   .F.
   DATA cNotifyIcon                               INIT   NIL
   DATA cNotifyTooltip                            INIT   NIL
   DATA bOnNotifyClick                            INIT   {|| .F. }
   DATA nVirtualHeight                            INIT   NIL
   DATA nVirtualWidth                             INIT   NIL
   DATA lHelpButton                               INIT   .F.
   DATA bOnInteractiveClose                       INIT   NIL
   DATA nType                                     INIT   0
   DATA lVirtualSized                             INIT   .F.
   DATA lAutoRelease                              INIT   .T.
   DATA oCurrentTabObject                         INIT   NIL
   DATA nCurrentTabPage                           INIT   0
   DATA oCurrentMenu                              INIT   NIL

   DATA oGraphLabel                               INIT   NIL

   // Properties: please preserve alphabetic order
   METHOD AutoRelease                             SETGET
   METHOD MaxButton                               SETGET
   METHOD MinButton                               SETGET
   METHOD NotifyIcon                              SETGET
   METHOD NotifyToolTip                           SETGET
   METHOD SysMenu                                 SETGET
   METHOD TitleBar                                SETGET
   METHOD Type                                    SETGET
   METHOD TopMost                                 SETGET
   METHOD VirtualWidth                            SETGET
   METHOD VirtualHeight                           SETGET
   METHOD HelpButton                              SETGET

   // Events: please preserve alphabetic order
   METHOD OnInteractiveClose                      SETGET
   METHOD OnNotifyClick                           SETGET
   METHOD OnRelease                               SETGET

   // Methods: please preserve alphabetic order. New and Create method must be the first if present
   METHOD New
   METHOD Create
   METHOD Activate
   METHOD EndWindow
   METHOD Restore
   METHOD SetFocus
   METHOD ShowNotifyMessage
   METHOD SetUpMenu

ENDCLASS

/*----------------------------------------------------------------------*/
//                              Properties
/*----------------------------------------------------------------------*/

METHOD NotifyIcon( cValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::cNotifyIcon
   ELSEIF VALTYPE( cValue ) == "C"
      ::cNotifyIcon := cValue
      IF VALTYPE( ::oSystemTrayIcon ) == "O"
         ::oSystemTrayIcon:setIcon( ::cNotifyIcon )
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD NotifyToolTip( cValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::cNotifyToolTip
   ELSEIF VALTYPE( cValue ) == "C"
      ::cNotifyToolTip := cValue
      IF VALTYPE( ::oSystemTrayIcon ) == "O"
         ::oSystemTrayIcon:setToolTip( ::cNotifyToolTip )
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD VirtualWidth( nValue ) CLASS WINDOW

   IF   PCOUNT() == 0
      RETURN ::nVirtualWidth
   ELSEIF VALTYPE( nValue ) == "N"
      ::nVirtualWidth := nValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD VirtualHeight( nValue ) CLASS WINDOW

   IF   PCOUNT() == 0
      RETURN ::nVirtualHeight
   ELSEIF VALTYPE( nValue ) == "N"
      ::nVirtualHeight := nValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Type( nValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::nType
   ELSEIF VALTYPE( nValue ) == "N"
      ::nType := nValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/


METHOD AutoRelease( lValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::lAutoRelease
   ELSEIF VALTYPE( lValue ) == "L"
      ::lAutoRelease := lValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD MinButton( lValue ) CLASS WINDOW
   LOCAL nFlags

   nFlags := ::oQTObject:windowFlags()
   IF PCOUNT() == 0
      IF hb_bitAnd( nFlags, Qt_WindowMinimizeButtonHint ) == Qt_WindowMinimizeButtonHint
         RETURN .T.
      ELSE
         RETURN .F.
      ENDIF
   ELSEIF VALTYPE( lValue ) == "L"
      ::lMinButton := lValue
      IF hb_bitAnd( nFlags, Qt_WindowMinimizeButtonHint ) == Qt_WindowMinimizeButtonHint // minimized button is inplace
         IF !lValue
            nFlags -= Qt_WindowMinimizeButtonHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ELSE
         IF lValue
            nFlags += Qt_WindowMinimizeButtonHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD MaxButton( lValue ) CLASS WINDOW
   LOCAL nFlags

   nFlags := ::oQTObject:windowFlags()
   IF PCOUNT() == 0
      IF hb_bitAnd( nFlags, Qt_WindowMaximizeButtonHint ) == Qt_WindowMaximizeButtonHint
         RETURN .t.
      ELSE
         RETURN .f.
      ENDIF
   ELSEIF VALTYPE( lValue ) == "L"
      IF hb_bitAnd( nFlags, Qt_WindowMaximizeButtonHint ) == Qt_WindowMaximizeButtonHint // minimized button is inplace
         IF !lValue
            nFlags -= Qt_WindowMaximizeButtonHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ELSE
         IF lValue
            nFlags += Qt_WindowMaximizeButtonHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD SysMenu( lValue ) CLASS WINDOW
   LOCAL nFlags

   nFlags := ::oQTObject:windowFlags()
   IF PCOUNT() == 0
      IF hb_bitAnd( nFlags, Qt_WindowSystemMenuHint ) == Qt_WindowSystemMenuHint
         RETURN .t.
      ELSE
         RETURN .f.
      ENDIF
   ELSEIF VALTYPE( lValue ) == "L"
      ::lSysMenu := lValue
      IF hb_bitAnd( nFlags, Qt_WindowSystemMenuHint ) == Qt_WindowSystemMenuHint // system menu is inplace
         IF !lValue
            nFlags += Qt_FramelessWindowHint
            nFlags -= Qt_WindowSystemMenuHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ELSE
         IF lValue
            nFlags -= Qt_FramelessWindowHint
            nFlags += Qt_WindowSystemMenuHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD TitleBar( lValue ) CLASS WINDOW
   LOCAL nFlags

   nFlags := ::oQTObject:windowFlags()
   IF PCOUNT() == 0
      IF hb_bitAnd( nFlags, Qt_FramelessWindowHint ) == Qt_FramelessWindowHint
         RETURN .F.
      ELSE
         RETURN .T.
      ENDIF
   ELSEIF VALTYPE( lValue ) == "L"
      ::lTitleBar := lValue
      IF hb_bitAnd( nFlags, Qt_FramelessWindowHint ) == Qt_FramelessWindowHint // titlebar is not inplace
         IF ::lTitleBar == .T.
            nFlags -= Qt_FramelessWindowHint
            nFlags += Qt_WindowCloseButtonHint
            nFlags += Qt_WindowTitleHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ELSE
         IF ::lTitleBar == .F.
            IF hb_bitAnd( nFlags, Qt_WindowCloseButtonHint ) == Qt_WindowCloseButtonHint // close button is also there!
               nFlags -= Qt_WindowCloseButtonHint
            ENDIF
            IF hb_bitAnd( nFlags, Qt_WindowTitleHint ) == Qt_WindowTitleHint // title bar should be removed
               nFlags -= Qt_WindowTitleHint
            ENDIF
            nFlags += Qt_FramelessWindowHint  // frameless window flag is enabled
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD HelpButton( lValue ) CLASS WINDOW
   LOCAL nFlags

   nFlags := ::oQTObject:windowFlags()
   IF PCOUNT() == 0
      IF hb_bitAnd( nFlags, Qt_WindowContextHelpButtonHint ) == Qt_WindowContextHelpButtonHint
         RETURN .t.
      ELSE
         RETURN .f.
      ENDIF
   ELSEIF VALTYPE( lValue ) == "L"
      ::lHelpButton := lValue
      IF hb_bitAnd( nFlags, Qt_WindowContextHelpButtonHint ) == Qt_WindowContextHelpButtonHint // helpbutton is inplace
         IF ::lHelpButton == .F.
            nFlags -= Qt_WindowContextHelpButtonHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ELSE
         IF ::lHelpButton == .T.
            nFlags += Qt_WindowContextHelpButtonHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD TopMost( lValue ) CLASS WINDOW
   LOCAL nFlags

   nFlags := ::oQTObject:windowFlags()
   IF PCOUNT() == 0
      IF hb_bitAnd( nFlags, Qt_WindowStaysOnTopHint ) == Qt_WindowStaysOnTopHint
         RETURN .t.
      ELSE
         RETURN .f.
      ENDIF
   ELSEIF VALTYPE( lValue ) == "L"
      ::lTopMost := lValue
      IF hb_bitAnd( nFlags, Qt_WindowStaysOnTopHint ) == Qt_WindowStaysOnTopHint // already topmost
         IF ::lTopMost == .F.
            nFlags -= Qt_WindowStaysOnTopHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ELSE
         IF ::lTopMost == .T.
            nFlags += Qt_WindowStaysOnTopHint
            ::oQTObject:setWindowFlags( nFlags )
            ::oQTObject:show()
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/
//                               Events
/*----------------------------------------------------------------------*/

METHOD OnRelease( bValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::bOnRelease
   ELSEIF VALTYPE( bValue ) == "B"
      ::bOnRelease := bValue
      IF ::nType == WND_MAIN
         IF .NOT. ::oQTObject:connect( QEvent_Close , { ||( EVAL( ::bOnRelease ), HMGAPP():Quit() ) } )
            // Error
         ENDIF
      ELSEIF ::nType == WND_MODAL
         IF .NOT. ::oQTObject:connect( QEvent_Close , { ||( EVAL( ::bOnRelease ), ::oEventLoop:Exit( 0 ) )  } )
            // Error
         ENDIF
      ELSE
         IF .NOT. ::oQTObject:connect( QEvent_Close , { ||( EVAL( ::bOnRelease ) )  } )
            // Error
         ENDIF
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnInteractiveClose( bValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::bOnInteractiveClose
   ELSEIF VALTYPE( bValue ) == "B"
      ::bOnInteractiveClose := bValue
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD OnNotifyClick( bValue ) CLASS WINDOW

   IF PCOUNT() == 0
      RETURN ::bOnNotifyClick
   ELSEIF VALTYPE( bValue ) == "B"
      ::bOnNotifyClick := bValue
      IF VALTYPE( ::oSystemTrayIcon ) == "O"
         ::oSystemTrayIcon:connect( "activated(QSystemTrayIcon::ActivationReason)", {|p| if( p == 3 , EVAL( ::bOnNotifyClick ) , NIL ) } )
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/
//                                 Methods
/*----------------------------------------------------------------------*/

METHOD New( cName, oParent ) CLASS WINDOW

   // check if HMGAPP is started
   IF ::s_lQtStarted == .F.
      HMGAPP():Start()
   ENDIF

   IF PCOUNT() == 0 .OR. VALTYPE( cName ) <> "C"
      ::cName := '_HMG_OBJECT_' + hb_nTos( ::s_nObjectCounter++ )
   ELSE
      ::cName  := cName
   ENDIF

   IF hb_IsObject( oParent ) .AND. ;                // If parent object is passed explicitly, use it.
      oParent:cClass == "WINDOW"                    // A Window can be child of a Window not controls...

      ::oParent    := oParent
      AADD( ::oParent:aControls , Self )
      ::oParent:AddData( ::cName , Self )

   ELSE

      ::s_oDefaultParent   := Self
      ::oParent            := ::s_oTopParent       // his parent it's hmgapp
      AADD( ::oParent:aControls, Self )
      ::oParent:AddData( ::cName, Self )

   ENDIF

   // Create The Window
   ::oQTObject := QMainWindow()

   ::oFont := ::oQTObject:Font()

   RETURN Self

/*----------------------------------------------------------------------*/

METHOD Create CLASS Window

   IF ::lCreated
      RETURN NIL
   ENDIF

   ::lCreated := .T.

   // Only one WND_MAIN window is allowed.
   // The first one will be WND_MAIN and only the first.
   IF Hb_IsNil( ::s_oMainWindow )
      ::nType := WND_MAIN
      ::s_oMainWindow := ::oQTObject
   ELSEIF ::nType == WND_MAIN
      ::nType := WND_MODAL
   ENDIF

   // Customize According Window Type
   IF   ::nType == WND_STANDARD   // Standard
      // Nothing To Do
   ELSEIF   ::nType == WND_MAIN   // Main
      IF VALTYPE( ::bOnRelease ) <> "B"
         IF .NOT. ::oQTObject:connect( QEvent_Close , { || HMGAPP():Quit() } )
            // Error
         ENDIF
      ENDIF
   ELSEIF   ::nType == WND_CHILD   // Child
      // Not Implemented( Ignored )
   ELSEIF   ::nType == WND_PANEL   // Panel
      // Not Implemented( Ignored )
   ELSEIF   ::nType == WND_MODAL   // Modal
      ::oQTObject:SetWindowModality( 2 )
      IF VALTYPE( ::bOnRelease ) <> "B"
         IF .NOT. ::oQTObject:connect( QEvent_Close , { || ::oEventLoop:Exit( 0 ) } )
            // Error
         ENDIF
      ENDIF
   ELSEIF   ::nType == WND_SPLITCHILD   // SplitShild??????
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD SetUpMenu( oValue ) CLASS WINDOW

   IF PCOUNT() == 1 .AND. VALTYPE( oValue ) == "O"
      ::oMainMenu:= oValue
   ENDIF

RETURN SELF

/*----------------------------------------------------------------------*/

METHOD Restore CLASS Window

   ::oQTObject:showNormal()

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD SetFocus CLASS Window

   ::oQTObject:activateWindow()

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD EndWindow() CLASS Window

   ::s_oDefaultParent := NIL

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD Activate() CLASS Window

   LOCAL cObjName

   // Create Window
   IF !::lCreated
      Self:Create()
      ::lCreated := .T.
   ENDIF

   // FOR KeyBoardEventProcessing...
   ::oQTObject:connect( QEvent_KeyPress , {|e|Self:DoKeyBoardEvents( e )} )

   //For Add/Remove Window Focus - ThisWindow implementation
   ::oQTObject:connect( QEvent_FocusIn , {||Self:AddWindowFocus( ) } )

   ::CreatePendingChildControls()

   //This must be repeat to prevent use of THISWINDOW inside ONINT
   // I think WND_MAIN can't be invisible AND run QT even if is visible
   IF ::lVisible == .T. .OR. ::nType == WND_MAIN
      cObjName := ::cName
      ::s_oCurWindow  := ::oParent:&cObjName
   ENDIF

   // Execute OnInit Block
   IF VALTYPE( ::bOnInit ) == "B"
      EVAL( ::bOnInit )
   ENDIF

   // Start Message Loop
   ::lActivated := .T.

   // Set the focus policy to capture focus and keyboard events.
   IF ::oQTObject:focusPolicy() == Qt_NoFocus
      ::oQTObject:setFocusPolicy( Qt_StrongFocus )
   ENDIF

   // Window specific properties, events.
   IF VALTYPE( ::cFontName )     != 'U'; Self:FontName     := ::cFontName     ; ENDIF
   IF VALTYPE( ::nFontSize )     != 'U'; Self:FontSize     := ::nFontSize     ; ENDIF
   IF VALTYPE( ::bOnSize )       != 'U'; Self:OnSize       := ::bOnSize       ; ENDIF
   IF VALTYPE( ::bOnMinimize )   != 'U'; Self:OnMinimize   := ::bOnMinimize   ; ENDIF
   IF VALTYPE( ::bOnMaximize )   != 'U'; Self:OnMaximize   := ::bOnMaximize   ; ENDIF
   IF VALTYPE( ::bOnPaint )      != 'U'; Self:OnPaint      := ::bOnPaint      ; ENDIF
   IF VALTYPE( ::bOnGotFocus )   != 'U'; Self:OnGotFocus   := ::bOnGotFocus   ; ENDIF
   IF VALTYPE( ::bOnLostFocus )  != 'U'; Self:OnLostFocus  := ::bOnLostFocus  ; ENDIF

   // I think WND_MAIN can't be invisible AND run QT even if is visible
   IF ::lVisible == .T. .OR. ::nType == WND_MAIN
      ::oQTObject:Show()
      IF ::nType == WND_MAIN   //Main
         ::s_oQtApp:exec()
      ELSE
         ::oEventLoop := QEventLoop( ::oQTObject )
         ::oEventLoop:Exec( 0 )
      ENDIF
   ENDIF

   RETURN NIL

/*----------------------------------------------------------------------*/

METHOD ShowNotifyMessage( cCaption , cMessage , nIcon ) CLASS WINDOW

   ::oSystemTrayIcon:showMessage( cCaption , cMessage , nIcon , 10000 )

   RETURN Self

/*----------------------------------------------------------------------*/
